==> pftpd-0.18.tar.gz: pftpd-0.18/src/dirlist.c <==
/*
** dirlist.c - Get a directory listing
**
** Copyright (c) 1999 Peter Eriksson <pen@signum.se>
**
** This program is free software; you can redistribute it and/or
** modify it as you wish - as long as you don't claim that you wrote
** it.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
*/

#include "config.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "safeio.h"
#include "readdir.h"
#include "dirlist.h"

#define MAX_PATHNAMELEN 2048

int dirlist_use_lstat = 1;
extern int debug;


static int
dirent_compare(const void *e1,
	       const void *e2)
{
    const struct dirlist_ent *d1, *d2;
    

    d1 = (struct dirlist_ent *) e1;
    d2 = (struct dirlist_ent *) e2;

    if ((S_IFMT & d1->s.st_mode) == S_IFDIR && 
	(S_IFMT & d2->s.st_mode) != S_IFDIR)
	return -1;

    if ((S_IFMT & d1->s.st_mode) != S_IFDIR && 
	(S_IFMT & d2->s.st_mode) == S_IFDIR)
	return 1;
	
    return strcmp(d1->d->d_name, d2->d->d_name);
}


DIRLIST *
dirlist_get(const char *path)
{
    S_DIR *dp;
    DIRLIST *dlp;
    struct dirent *dep;
    char p_buf[MAX_PATHNAMELEN], *pp;
    int err;
    

    if (path == NULL)
	return NULL;
    
    dp = s_opendir(path);
    if (dp == NULL)
	return NULL;

    strcpy(p_buf, path);
    pp = p_buf+strlen(p_buf);
    if (pp > p_buf && pp[-1] != '/')
	*pp++ = '/';
    
    dlp = s_malloc(sizeof(*dlp));
    
    dlp->dec = 0;
    dlp->des = 64;
    dlp->dev = s_malloc(dlp->des * sizeof(dlp->dev[0]));

    while ((dep = s_readdir(dp)) != NULL)
    {
	if (dlp->dec == dlp->des - 1)
	{
	    dlp->des += 64;
	    dlp->dev = s_realloc(dlp->dev,
				 dlp->des * sizeof(dlp->dev[0]));
	}

	dlp->dev[dlp->dec].d = dep;

	strcpy(pp, dep->d_name);
	if (dirlist_use_lstat)
	    err = lstat(p_buf, &dlp->dev[dlp->dec].s);
	else
	    err = stat(p_buf, &dlp->dev[dlp->dec].s);

	if (err)
	    memset(&dlp->dev[dlp->dec].s, 0, sizeof(dlp->dev[dlp->dec].s));
	
	dlp->dec++;
    }
    s_closedir(dp);
    dlp->dev[dlp->dec].d = NULL;

    qsort(dlp->dev, dlp->dec, sizeof(dlp->dev[0]), dirent_compare);

    return dlp;
}


void
dirlist_free(DIRLIST *dlp)
{
    int i;


    if (dlp == NULL)
	return;
    
    for (i = 0; i < dlp->dec; i++)
	s_free(dlp->dev[i].d);
    s_free(dlp->dev);
    s_free(dlp);
}


#ifdef TEST
int debug = 0;

int
main(int argc,
     char *argv[])
{
    DIRLIST *dlp;
    int i;
    

    dlp = dirlist_get(argv[1]);
    if (dlp == NULL)
    {
	perror("dirlist_get");
	exit(1);
    }

    printf("%d entries:\n", dlp->dec);
    
    for (i = 0; i < dlp->dec; i++)
	puts(dlp->dev[i].d->d_name);

    dirlist_free(dlp);
}
#endif

==> pftpd-0.20.tar.gz: pftpd-0.20/src/dirlist.c <==
/*
** dirlist.c - Get a directory listing
**
** Copyright (c) 1999 Peter Eriksson <pen@signum.se>
**
** This program is free software; you can redistribute it and/or
** modify it as you wish - as long as you don't claim that you wrote
** it.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
*/

#include "config.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "safeio.h"
#include "readdir.h"
#include "dirlist.h"

#define MAX_PATHNAMELEN 2048

int dirlist_use_lstat = 1;
extern int debug;


static int
dirent_compare(const void *e1,
	       const void *e2)
{
    const struct dirlist_ent *d1, *d2;
    

    d1 = (struct dirlist_ent *) e1;
    d2 = (struct dirlist_ent *) e2;

    if ((S_IFMT & d1->s.st_mode) == S_IFDIR && 
	(S_IFMT & d2->s.st_mode) != S_IFDIR)
	return -1;

    if ((S_IFMT & d1->s.st_mode) != S_IFDIR && 
	(S_IFMT & d2->s.st_mode) == S_IFDIR)
	return 1;
	
    return strcmp(d1->d->d_name, d2->d->d_name);
}


DIRLIST *
dirlist_get(const char *path)
{
    S_DIR *dp;
    DIRLIST *dlp;
    struct dirent *dep;
    char p_buf[MAX_PATHNAMELEN], *pp;
    int err;
    

    if (path == NULL)
	return NULL;
    
    dp = s_opendir(path);
    if (dp == NULL)
	return NULL;

    if (s_strcpy(p_buf, sizeof(p_buf)-1, path) == NULL)
	return NULL;
    
    pp = p_buf+strlen(p_buf);
    if (pp > p_buf && pp[-1] != '/')
	*pp++ = '/';
    
    dlp = s_malloc(sizeof(*dlp));
    
    dlp->dec = 0;
    dlp->des = 64;
    dlp->dev = s_malloc(dlp->des * sizeof(dlp->dev[0]));

    while ((dep = s_readdir(dp)) != NULL)
    {
	if (dlp->dec == dlp->des - 1)
	{
	    dlp->des += 64;
	    dlp->dev = s_realloc(dlp->dev,
				 dlp->des * sizeof(dlp->dev[0]));
	}

	dlp->dev[dlp->dec].d = dep;

	s_strcpy(pp, sizeof(p_buf)-(pp-p_buf), dep->d_name);
	
	if (dirlist_use_lstat)
	    err = lstat(p_buf, &dlp->dev[dlp->dec].s);
	else
	    err = stat(p_buf, &dlp->dev[dlp->dec].s);

	if (err)
	    memset(&dlp->dev[dlp->dec].s, 0, sizeof(dlp->dev[dlp->dec].s));
	
	dlp->dec++;
    }
    s_closedir(dp);
    dlp->dev[dlp->dec].d = NULL;

    qsort(dlp->dev, dlp->dec, sizeof(dlp->dev[0]), dirent_compare);

    return dlp;
}


void
dirlist_free(DIRLIST *dlp)
{
    int i;


    if (dlp == NULL)
	return;
    
    for (i = 0; i < dlp->dec; i++)
	s_free(dlp->dev[i].d);
    s_free(dlp->dev);
    s_free(dlp);
}


#ifdef TEST
int debug = 0;

int
main(int argc,
     char *argv[])
{
    DIRLIST *dlp;
    int i;
    

    dlp = dirlist_get(argv[1]);
    if (dlp == NULL)
    {
	perror("dirlist_get");
	exit(1);
    }

    printf("%d entries:\n", dlp->dec);
    
    for (i = 0; i < dlp->dec; i++)
	puts(dlp->dev[i].d->d_name);

    dirlist_free(dlp);
}
#endif
